diff -Nru linux-2.6.9.notape/drivers/scsi/st.c linux-2.6.9/drivers/scsi/st.c
--- linux-2.6.9.notape/drivers/scsi/st.c	2006-06-27 14:47:29.000000000 +0200
+++ linux-2.6.9/drivers/scsi/st.c	2006-06-27 14:55:35.000000000 +0200
@@ -429,6 +429,31 @@
 		SRpnt->sr_request->waiting = NULL;
 		(STp->buffer)->syscall_result = st_chk_result(STp, SRpnt);
 	}
+
+	/* Report sense bytes and residue information to the application.
+	 * Some sr_sense_buffer bytes (as explained by Jean-Philippe Baud):
+	 * 	[0]: leftmost bit: validity of residual count
+	 * 	[2] (low 4 bits): sense key, SCSI-standard errors
+	 * 		e.g. media error, blank tape, hardware error, etc.
+	 * 	{3-6]: residual count (32-bit), meaningful only if validity is 1
+	 * 	[12] (ASC): manufacturer-dependant error report byte, some
+	 * 		values are valid across manufacturers, some are not.
+	 * 	[13]: even more manufacturer-dependant, but more detailed.
+	 * Originally by Fabien Collin. */
+	STp->sense_bytes =
+		  SRpnt->sr_sense_buffer[2]	<< 16
+		| SRpnt->sr_sense_buffer[12]	<< 8
+		| SRpnt->sr_sense_buffer[13];
+	if (SRpnt->sr_sense_buffer[0] & 0x80) {
+		STp->residu = 
+			  SRpnt->sr_sense_buffer[3]	<< 24
+			| SRpnt->sr_sense_buffer[4]	<< 16
+			| SRpnt->sr_sense_buffer[5]	<< 8
+			| SRpnt->sr_sense_buffer[6];
+	} else {
+		STp->residu = 0;
+	}
+
 	return SRpnt;
 }
 
@@ -3354,8 +3379,17 @@
 			else
 				mt_status.mt_gstat |= GMT_EOF(0xffffffff);
 		}
+		/* Hijack partition support (since currently it's
+		 * implemented on DAT only anyway) to report residue info
+		 * instead.
+		 * Originally by Fabien Collin.
 		mt_status.mt_erreg = (STp->recover_reg << MT_ST_SOFTERR_SHIFT);
 		mt_status.mt_resid = STp->partition;
+		*/
+
+		mt_status.mt_erreg = STp->sense_bytes;
+		mt_status.mt_resid = STp->residu;
+
 		if (STps->eof == ST_EOM_OK || STps->eof == ST_EOM_ERROR)
 			mt_status.mt_gstat |= GMT_EOT(0xffffffff);
 		else if (STps->eof >= ST_EOM_OK)
diff -Nru linux-2.6.9.notape/drivers/scsi/st.h linux-2.6.9/drivers/scsi/st.h
--- linux-2.6.9.notape/drivers/scsi/st.h	2004-10-18 23:54:55.000000000 +0200
+++ linux-2.6.9/drivers/scsi/st.h	2006-06-27 14:53:27.000000000 +0200
@@ -133,6 +133,8 @@
 	int max_block;
 	int recover_count;     /* From tape opening */
 	int recover_reg;       /* From last status call */
+	int sense_bytes;
+	int residu;
 
 #if DEBUG
 	unsigned char write_pending;
diff -Nru linux-2.6.9.notape/drivers/scsi/st_options.h linux-2.6.9/drivers/scsi/st_options.h
--- linux-2.6.9.notape/drivers/scsi/st_options.h	2004-10-18 23:54:37.000000000 +0200
+++ linux-2.6.9/drivers/scsi/st_options.h	2006-06-27 14:53:27.000000000 +0200
@@ -63,16 +63,19 @@
 /* If ST_BUFFER_WRITES is non-zero, writes in fixed block mode are
    buffered until the driver buffer is full or asynchronous write is
    triggered. May make detection of End-Of-Medium early enough fail. */
-#define ST_BUFFER_WRITES 1
+/* Set to FALSE to have better error reporting.  Originally by Fabien Collin. */
+#define ST_BUFFER_WRITES 0
 
 /* If ST_ASYNC_WRITES is non-zero, the SCSI write command may be started
    without waiting for it to finish. May cause problems in multiple
    tape backups. */
-#define ST_ASYNC_WRITES 1
+/* Set to FALSE to have sense bytes reporting working. Originally by Fabien Collin. */
+#define ST_ASYNC_WRITES 0
 
 /* If ST_READ_AHEAD is non-zero, blocks are read ahead in fixed block
    mode. */
-#define ST_READ_AHEAD 1
+/* Set to FALSE (particular reason unknown). Originally by Fabien Collin. */
+#define ST_READ_AHEAD 0
 
 /* If ST_AUTO_LOCK is non-zero, the drive door is locked at the first
    read or write command after the device is opened. The door is opened
